@@ -38,10 +38,10 @@ using namespace iff_pvt;
 // Obligatory material to make this a recognizeable imageio plugin
 OIIO_PLUGIN_EXPORTS_BEGIN
 
-    DLLEXPORT ImageOutput *iff_output_imageio_create () {
+    OIIO_EXPORT ImageOutput *iff_output_imageio_create () {
         return new IffOutput;
     }
-    DLLEXPORT const char *iff_output_extensions[] = {
+    OIIO_EXPORT const char *iff_output_extensions[] = {
         "iff", "z", NULL
     };
 
@@ -101,7 +101,7 @@ IffOutput::open (const std::string &name, const ImageSpec &spec,
     m_spec.tile_width = tile_width();
     m_spec.tile_height = tile_height();
     
-    m_fd = fopen (m_filename.c_str (), "wb");
+    m_fd = Filesystem::fopen (m_filename, "wb");
     if (!m_fd) {
         error ("Unable to open file \"%s\"", m_filename.c_str ());
         return false;
