@@ -160,7 +160,7 @@ JpgOutput::open (const std::string &name, const ImageSpec &newspec,
     m_cinfo.density_unit = 2; // RESUNIT_INCH;
     m_cinfo.X_density = 72;
     m_cinfo.Y_density = 72;
-    m_cinfo.write_JFIF_header = true;
+    m_cinfo.write_JFIF_header = TRUE;
 
     if (m_copy_coeffs) {
         // Back door for copy()
@@ -228,7 +228,7 @@ JpgOutput::open (const std::string &name, const ImageSpec &newspec,
     if (! xmp.empty()) {
         static char prefix[] = "http://ns.adobe.com/xap/1.0/";
         std::vector<char> block (prefix, prefix+strlen(prefix)+1);
-        block.insert (block.end(), xmp.c_str(), xmp.c_str()+xmp.length()+1);
+        block.insert (block.end(), xmp.c_str(), xmp.c_str()+xmp.length());
         jpeg_write_marker (&m_cinfo, JPEG_APP0+1, (JOCTET*)&block[0], block.size());
     }
 
