@@ -35,6 +35,7 @@
 #include "strutil.h"
 #include "color.h"
 #include "imagebufalgo.h"
+#include "imagebufalgo_util.h"
 
 #ifdef USE_OCIO
 #include <OpenColorIO/OpenColorIO.h>
@@ -195,10 +196,34 @@ ColorConfig::getColorSpaceNameByIndex (int index) const
 
 
 
+int
+ColorConfig::getNumLooks () const
+{
+#ifdef USE_OCIO
+    if (getImpl()->config_)
+        return getImpl()->config_->getNumLooks();
+#endif
+    return 0;
+}
+
+
+
+const char *
+ColorConfig::getLookNameByIndex (int index) const
+{
+#ifdef USE_OCIO
+    if (getImpl()->config_)
+        return getImpl()->config_->getLookNameByIndex (index);
+#endif
+    return NULL;
+}
+
+
+
 const char *
 ColorConfig::getColorSpaceNameByRole (const char *role) const
 {
-#if USE_OCIO
+#ifdef USE_OCIO
     if (getImpl()->config_) {
         OCIO::ConstColorSpaceRcPtr c = getImpl()->config_->getColorSpace (role);
         // Catch special case of obvious name synonyms
@@ -220,6 +245,78 @@ ColorConfig::getColorSpaceNameByRole (const char *role) const
 
 
 
+int
+ColorConfig::getNumDisplays() const
+{
+#ifdef USE_OCIO
+    if (getImpl()->config_)
+        return getImpl()->config_->getNumDisplays();
+#endif
+    return 0;
+}
+
+
+
+const char *
+ColorConfig::getDisplayNameByIndex(int index) const
+{
+#ifdef USE_OCIO
+    if (getImpl()->config_)
+        return getImpl()->config_->getDisplay(index);
+#endif
+    return NULL;
+}
+
+
+
+int
+ColorConfig::getNumViews(const char * display) const
+{
+#ifdef USE_OCIO
+    if (getImpl()->config_)
+        return getImpl()->config_->getNumViews(display);
+#endif
+    return 0;
+}
+
+
+
+const char *
+ColorConfig::getViewNameByIndex(const char * display, int index) const
+{
+#ifdef USE_OCIO
+    if (getImpl()->config_)
+        return getImpl()->config_->getView(display, index);
+#endif
+    return NULL;
+}
+
+
+
+const char *
+ColorConfig::getDefaultDisplayName() const
+{
+#ifdef USE_OCIO
+    if (getImpl()->config_)
+        return getImpl()->config_->getDefaultDisplay();
+#endif
+    return NULL;
+}
+
+
+
+const char *
+ColorConfig::getDefaultViewName(const char * display) const
+{
+#ifdef USE_OCIO
+    if (getImpl()->config_)
+        return getImpl()->config_->getDefaultView(display);
+#endif
+    return NULL;
+}
+
+
+
 // Abstract wrapper class for objects that will apply color transformations.
 class ColorProcessor
 {
@@ -262,6 +359,7 @@ private:
 #endif
 
 
+
 // ColorProcessor that hard-codes sRGB-to-linear
 class ColorProcessor_sRGB_to_linear : public ColorProcessor {
 public:
@@ -278,7 +376,7 @@ public:
             char *d = (char *)data + y*ystride;
             for (int x = 0;  x < width;  ++x, d += xstride)
                 for (int c = 0;  c < channels;  ++c)
-                    *(float *)d = sRGB_to_linear (*(float *)d);
+                    ((float *)d)[c] = sRGB_to_linear (((float *)d)[c]);
         }
     }
 };
@@ -323,7 +421,7 @@ public:
             char *d = (char *)data + y*ystride;
             for (int x = 0;  x < width;  ++x, d += xstride)
                 for (int c = 0;  c < channels;  ++c)
-                    *(float *)d = Rec709_to_linear (*(float *)d);
+                    ((float *)d)[c] = Rec709_to_linear (((float *)d)[c]);
         }
     }
 };
@@ -405,6 +503,121 @@ ColorConfig::createColorProcessor (const char * inputColorSpace,
 
 
 
+ColorProcessor*
+ColorConfig::createLookTransform (const char * looks,
+                                  const char * inputColorSpace,
+                                  const char * outputColorSpace,
+                                  bool inverse,
+                                  const char *context_key,
+                                  const char *context_val) const
+{
+#ifdef USE_OCIO
+    // Ask OCIO to make a Processor that can handle the requested
+    // transformation.
+    if (getImpl()->config_) {
+        OCIO::ConstConfigRcPtr config = getImpl()->config_;
+        OCIO::LookTransformRcPtr transform = OCIO::LookTransform::Create();
+        transform->setLooks (looks);
+        OCIO::TransformDirection dir;
+        if (inverse) {
+            // The TRANSFORM_DIR_INVERSE applies an inverse for the
+            // end-to-end transform, which would otherwise do dst->inv
+            // look -> src.  This is an unintuitive result for the
+            // artist (who would expect in, out to remain unchanged), so
+            // we account for that here by flipping src/dst
+            transform->setSrc (outputColorSpace);
+            transform->setDst (inputColorSpace);
+            dir = OCIO::TRANSFORM_DIR_INVERSE;
+        } else { // forward
+            transform->setSrc (inputColorSpace);
+            transform->setDst (outputColorSpace);
+            dir = OCIO::TRANSFORM_DIR_FORWARD;
+        }
+        OCIO::ConstContextRcPtr context = config->getCurrentContext();
+        if (context_key && context_key[0] && context_val && context_val[0]) {
+            OCIO::ContextRcPtr ctx = context->createEditableCopy();
+            ctx->setStringVar (context_key, context_val);
+            context = ctx;
+        }
+
+        OCIO::ConstProcessorRcPtr p;
+        try {
+            // Get the processor corresponding to this transform.
+            p = getImpl()->config_->getProcessor (context, transform, dir);
+        }
+        catch(OCIO::Exception &e) {
+            getImpl()->error_ = e.what();
+            return NULL;
+        }
+        catch(...) {
+            getImpl()->error_ = "An unknown error occurred in OpenColorIO, getProcessor";
+            return NULL;
+        }
+    
+        getImpl()->error_ = "";
+        return new ColorProcessor_OCIO(p);
+    }
+#endif
+
+    return NULL;    // if we get this far, we've failed
+}
+
+
+
+ColorProcessor*
+ColorConfig::createDisplayTransform (const char * display,
+                                     const char * view,
+                                     const char * inputColorSpace,
+                                     const char * looks,
+                                     const char * context_key,
+                                     const char * context_value) const
+{
+#ifdef USE_OCIO
+    // Ask OCIO to make a Processor that can handle the requested
+    // transformation.
+    if (getImpl()->config_) {
+        OCIO::ConstConfigRcPtr config = getImpl()->config_;
+        OCIO::DisplayTransformRcPtr transform = OCIO::DisplayTransform::Create();
+        transform->setInputColorSpaceName (inputColorSpace);
+        transform->setDisplay(display);
+        transform->setView(view);
+        if (looks) {
+            transform->setLooksOverride(looks);
+            transform->setLooksOverrideEnabled(true);
+        } else {
+            transform->setLooksOverrideEnabled(false);
+        }
+        OCIO::ConstContextRcPtr context = config->getCurrentContext();
+        if (context_key && context_key[0] && context_value && context_value[0]) {
+            OCIO::ContextRcPtr ctx = context->createEditableCopy();
+            ctx->setStringVar (context_key, context_value);
+            context = ctx;
+        }
+
+        OCIO::ConstProcessorRcPtr p;
+        try {
+            // Get the processor corresponding to this transform.
+            p = getImpl()->config_->getProcessor (context, transform,
+                                                  OCIO::TRANSFORM_DIR_FORWARD);
+        }
+        catch(OCIO::Exception &e) {
+            getImpl()->error_ = e.what();
+            return NULL;
+        }
+        catch(...) {
+            getImpl()->error_ = "An unknown error occurred in OpenColorIO, getProcessor";
+            return NULL;
+        }
+    
+        getImpl()->error_ = "";
+        return new ColorProcessor_OCIO(p);
+    }
+#endif
+
+    return NULL;    // if we get this far, we've failed
+}
+
+
 void
 ColorConfig::deleteColorProcessor (ColorProcessor * processor)
 {
@@ -418,30 +631,89 @@ ColorConfig::deleteColorProcessor (ColorProcessor * processor)
 // Image Processing Implementations
 
 
+static boost::shared_ptr<ColorConfig> default_colorconfig;  // default color config
+static spin_mutex colorconfig_mutex;
+
+
+
 bool
 ImageBufAlgo::colorconvert (ImageBuf &dst, const ImageBuf &src,
-                            const ColorProcessor* processor, bool unpremult)
+                            const char *from, const char *to,
+                            bool unpremult, ROI roi, int nthreads)
 {
+    if (!from || !from[0] || !strcmp(from, "current")) {
+        std::string s = src.spec().get_string_attribute ("oiio:Colorspace", "Linear");
+        from = ustring(s).c_str();
+    }
+    if (!from || !to) {
+        dst.error ("Unknown color space name");
+        return false;
+    }
+    ColorConfig *config = NULL;
+    ColorProcessor *processor = NULL;
+    {
+        spin_lock lock (colorconfig_mutex);
+        config = default_colorconfig.get();
+        if (! config)
+            default_colorconfig.reset (config = new ColorConfig);
+        processor = config->createColorProcessor (from, to);
+        if (! processor) {
+            if (config->error())
+                dst.error ("%s", config->geterror());
+            else
+                dst.error ("Could not construct the color transform");
+            return false;
+        }
+    }
+    bool ok = colorconvert (dst, src, processor, unpremult, roi, nthreads);
+    if (ok)
+        dst.specmod().attribute ("oiio:ColorSpace", to);
+    config->deleteColorProcessor (processor);
+    return ok;
+}
+
+
+
+bool
+ImageBufAlgo::colorconvert (ImageBuf &dst, const ImageBuf &src,
+                            const ColorProcessor* processor, bool unpremult,
+                            ROI roi, int nthreads)
+{
+    // FIXME -- currently, we do not split across threads.
+
     // If the processor is NULL, return false (error)
     if (!processor) {
         dst.error ("Passed NULL ColorProcessor to colorconvert() [probable application bug]");
         return false;
     }
 
-    // If the processor is a no-op, no work needs to be done. Early exit.
-    if (processor->isNoOp())
+    // If the processor is a no-op and the conversion is being done
+    // in place, no work needs to be done. Early exit.
+    if (processor->isNoOp() && (&dst == &src))
         return true;
-    
-    const ImageSpec &dstspec = dst.spec();
+
+    IBAprep (roi, &dst, &src);
+
+    // If the processor is a no-op (and it's not an in-place conversion),
+    // use paste() to simplify the operation.
+    if (processor->isNoOp()) {
+        roi.chend = std::max (roi.chbegin+4, roi.chend);
+        return ImageBufAlgo::paste (dst, roi.xbegin, roi.ybegin, roi.zbegin,
+                                    roi.chbegin, src, roi, nthreads);
+    }
+
+    int width = roi.width();
     // Temporary space to hold one RGBA scanline
-    std::vector<float> scanline(dstspec.width*4, 0.0f);
+    std::vector<float> scanline(width*4, 0.0f);
     
-    // Only process up to, and including, the first 4 channels.
-    // This does let us process images with fewer than 4 channels, which is the intent
-    // FIXME: Instead of loading the first 4 channels, obey dstspec.alpha_channel index
-    //        (but first validate that the index is set properly for normal formats)
+    // Only process up to, and including, the first 4 channels.  This
+    // does let us process images with fewer than 4 channels, which is
+    // the intent.
+    // FIXME: Instead of loading the first 4 channels, obey
+    //        dstspec.alpha_channel index (but first validate that the
+    //        index is set properly for normal formats)
     
-    int channelsToCopy = std::min (4, dstspec.nchannels);
+    int channelsToCopy = std::min (4, roi.nchannels());
     
     // Walk through all data in our buffer. (i.e., crop or overscan)
     // FIXME: What about the display window?  Should this actually promote
@@ -462,25 +734,22 @@ ImageBufAlgo::colorconvert (ImageBuf &dst, const ImageBuf &src,
     bool clearScanline = (channelsToCopy<4 && 
                           (processor->hasChannelCrosstalk() || unpremult));
     
-    for (int k = dstspec.z; k < dstspec.z+dstspec.depth; k++) {
-        for (int j = dstspec.y; j <  dstspec.y+dstspec.height; j++) {
+    for (int k = roi.zbegin; k < roi.zend; ++k) {
+        for (int j = roi.ybegin; j < roi.yend; ++j) {
             // Clear the scanline
-            if (clearScanline) {
+            if (clearScanline)
                 memset (&scanline[0], 0, sizeof(float)*scanline.size());
-            }
             
             // Load the scanline
-            dstPtr = &scanline[0];
-            for (int i = dstspec.x; i < dstspec.x+dstspec.width ; i++) {
-                src.getpixel (i, j, dstPtr, channelsToCopy);
-                dstPtr += 4;
-            }
+            src.get_pixel_channels (roi.xbegin, roi.xend, j, j+1, k, k+1,
+                                    0, channelsToCopy,
+                                TypeDesc::TypeFloat, &scanline[0],
+                                4*sizeof(float));
             
             // Optionally unpremult
-            if ((channelsToCopy>=4) && unpremult) {
-                float alpha = 0.0;
-                for (int i=0; i<dstspec.width; ++i) {
-                    alpha = scanline[4*i+3];
+            if ((channelsToCopy >= 4) && unpremult) {
+                for (int i = 0; i < width; ++i) {
+                    float alpha = scanline[4*i+3];
                     if (alpha > fltmin) {
                         scanline[4*i+0] /= alpha;
                         scanline[4*i+1] /= alpha;
@@ -490,15 +759,14 @@ ImageBufAlgo::colorconvert (ImageBuf &dst, const ImageBuf &src,
             }
             
             // Apply the color transformation in place
-            processor->apply (&scanline[0], dstspec.width, 1, 4,
+            processor->apply (&scanline[0], width, 1, 4,
                               sizeof(float), 4*sizeof(float),
-                              dstspec.width*4*sizeof(float));
+                              width*4*sizeof(float));
             
             // Optionally premult
-            if ((channelsToCopy>=4) && unpremult) {
-                float alpha = 0.0;
-                for (int i=0; i<dstspec.width; ++i) {
-                    alpha = scanline[4*i+3];
+            if ((channelsToCopy >= 4) && unpremult) {
+                for (int i = 0; i < width; ++i) {
+                    float alpha = scanline[4*i+3];
                     if (alpha > fltmin) {
                         scanline[4*i+0] *= alpha;
                         scanline[4*i+1] *= alpha;
@@ -506,11 +774,11 @@ ImageBufAlgo::colorconvert (ImageBuf &dst, const ImageBuf &src,
                     }
                 }
             }
-            
+
             // Store the scanline
             dstPtr = &scanline[0];
-            for (int i = dstspec.x; i < dstspec.x+dstspec.width ; i++) {
-                dst.setpixel (i, j, dstPtr, channelsToCopy);
+            for (int i = roi.xbegin; i < roi.xend ; i++) {
+                dst.setpixel (i, j, k, dstPtr, channelsToCopy);
                 dstPtr += 4;
             }
         }
@@ -520,7 +788,91 @@ ImageBufAlgo::colorconvert (ImageBuf &dst, const ImageBuf &src,
 }
 
 
+
+bool
+ImageBufAlgo::ociolook (ImageBuf &dst, const ImageBuf &src,
+                        const char *looks, const char *from, const char *to,
+                        bool inverse, bool unpremult,
+                        const char *key, const char *value,
+                        ROI roi, int nthreads)
+{
+    if (!from || !from[0] || !strcmp(from, "current")) {
+        std::string s = src.spec().get_string_attribute ("oiio:Colorspace", "Linear");
+        from = ustring(s).c_str();
+    }
+    if (!to || !to[0] || !strcmp(to, "current")) {
+        std::string s = src.spec().get_string_attribute ("oiio:Colorspace", "Linear");
+        to = ustring(s).c_str();
+    }
+    if (!from || !to) {
+        dst.error ("Unknown color space name");
+        return false;
+    }
+    ColorConfig *config = NULL;
+    ColorProcessor *processor = NULL;
+    {
+        spin_lock lock (colorconfig_mutex);
+        config = default_colorconfig.get();
+        if (! config)
+            default_colorconfig.reset (config = new ColorConfig);
+        processor = config->createLookTransform (looks, from, to, inverse,
+                                                 key, value);
+        if (! processor) {
+            if (config->error())
+                dst.error ("%s", config->geterror());
+            else
+                dst.error ("Could not construct the color transform");
+            return false;
+        }
+    }
+    bool ok = colorconvert (dst, src, processor, unpremult, roi, nthreads);
+    if (ok)
+        dst.specmod().attribute ("oiio:ColorSpace", to);
+    config->deleteColorProcessor (processor);
+    return ok;
+}
+
+
     
+bool
+ImageBufAlgo::ociodisplay (ImageBuf &dst, const ImageBuf &src,
+                           const char *display, const char *view,
+                           const char *from, const char *looks,
+                           bool unpremult,
+                           const char *key, const char *value,
+                           ROI roi, int nthreads)
+{
+    if (!from || !from[0] || !strcmp(from, "current")) {
+        std::string s = src.spec().get_string_attribute ("oiio:Colorspace", "Linear");
+        from = ustring(s).c_str();
+    }
+    if (!from) {
+        dst.error ("Unknown color space name");
+        return false;
+    }
+    ColorConfig *config = NULL;
+    ColorProcessor *processor = NULL;
+    {
+        spin_lock lock (colorconfig_mutex);
+        config = default_colorconfig.get();
+        if (! config)
+            default_colorconfig.reset (config = new ColorConfig);
+        processor = config->createDisplayTransform (display, view, from, looks, key, value);
+        if (! processor) {
+            if (config->error())
+                dst.error ("%s", config->geterror());
+            else
+                dst.error ("Could not construct the color transform");
+            return false;
+        }
+    }
+    bool ok = colorconvert (dst, src, processor, unpremult, roi, nthreads);
+    config->deleteColorProcessor (processor);
+    return ok;
+}
+
+
+
 bool
 ImageBufAlgo::colorconvert (float * color, int nchannels,
                             const ColorProcessor* processor, bool unpremult)
@@ -572,5 +924,6 @@ ImageBufAlgo::colorconvert (float * color, int nchannels,
 }
 
 
+
 }
 OIIO_NAMESPACE_EXIT
