@@ -33,8 +33,6 @@
 #include <string>
 #include <sstream>
 #include <list>
-#include <boost/tr1/memory.hpp>
-using namespace std::tr1;
 
 #include <OpenEXR/ImathMatrix.h>
 
@@ -237,7 +235,7 @@ namespace {  // anonymous
 
 static EightBitConverter<float> uchar2float;
 
-};  // end anonymous namespace
+}  // end anonymous namespace
 
 namespace pvt {   // namespace pvt
 
@@ -252,11 +250,17 @@ TextureSystemImpl::environment (ustring filename, TextureOptions &options,
                                 VaryingRef<Imath::V3f> dRdy,
                                 float *result)
 {
+    Perthread *thread_info = get_perthread_info();
+    TextureHandle *texture_handle = get_texture_handle (filename, thread_info);
+    if (! texture_handle)
+        return false;
     bool ok = true;
     for (int i = beginactive;  i < endactive;  ++i) {
         if (runflags[i]) {
             TextureOpt opt (options, i);
-            ok &= environment (filename, opt, R[i], dRdx[i], dRdy[i], result);
+            ok &= environment (texture_handle, thread_info,
+                               opt, R[i], dRdx[i], dRdy[i],
+                               result+i*options.nchannels);
         }
     }
     return ok;
@@ -509,7 +513,7 @@ TextureSystemImpl::environment (TextureHandle *texture_handle_,
 
 
 
-};  // end namespace pvt
+}  // end namespace pvt
 
 }
 OIIO_NAMESPACE_EXIT
