@@ -30,8 +30,6 @@
 
 #include <string>
 #include <boost/scoped_ptr.hpp>
-#include <boost/tr1/memory.hpp>
-using namespace std::tr1;
 
 #include "dassert.h"
 #include "typedesc.h"
@@ -64,7 +62,7 @@ static const ustring wrap_type_name[] = {
     ustring()
 };
 
-};  // end anonymous namespace
+}  // end anonymous namespace
 
 
 
