@@ -274,17 +274,10 @@ private:
     /// Find the TextureFile record for the named texture, or NULL if no
     /// such file can be found.
     TextureFile *find_texturefile (ustring filename, PerThreadInfo *thread_info) {
-        // Per-thread microcache that prevents locking of the file mutex
-        TextureFile *texturefile = thread_info->find_file (filename);
-        if (! texturefile) {
-            // Fall back on the master cache
-            texturefile = m_imagecache->find_file (filename, thread_info);
-            if (!texturefile || texturefile->broken())
-                error ("%s", m_imagecache->geterror().c_str());
-            thread_info->filename (filename, texturefile);
-        }
+        TextureFile *texturefile = m_imagecache->find_file (filename, thread_info);
+        if (!texturefile || texturefile->broken())
+            error ("%s", m_imagecache->geterror().c_str());
         return texturefile;
-
     }
 
     /// Find the tile specified by id.  If found, return true and place
@@ -416,13 +409,7 @@ private:
     /// Handle gray-to-RGB promotion.
     void fill_gray_channels (const ImageSpec &spec, TextureOpt &options, float *result);
 
-    typedef bool (*wrap_impl) (int &coord, int origin, int width);
-    static bool wrap_black (int &coord, int origin, int width);
-    static bool wrap_clamp (int &coord, int origin, int width);
-    static bool wrap_periodic (int &coord, int origin, int width);
-    static bool wrap_periodic2 (int &coord, int origin, int width);
     static bool wrap_periodic_sharedborder (int &coord, int origin, int width);
-    static bool wrap_mirror (int &coord, int origin, int width);
     static const wrap_impl wrap_functions[];
 
     /// Helper function for lat-long environment maps: compute a "pole"
@@ -470,8 +457,7 @@ private:
     mutable thread_specific_ptr< std::string > m_errormessage;
     Filter1D *hq_filter;         ///< Better filter for magnification
     int m_statslevel;
-    friend class ImageCacheFile;
-    friend class ImageCacheTile;
+    friend class TextureSystem;
 };
 
 
@@ -547,7 +533,7 @@ TextureSystemImpl::st_to_texel (float s, float t, TextureFile &texturefile,
 
 
 
-};  // end namespace pvt
+}  // end namespace pvt
 
 }
 OIIO_NAMESPACE_EXIT
