@@ -322,7 +322,6 @@ ArgOption::add_argument (const char *argv)
 ArgParse::ArgParse (int argc, const char **argv)
     : m_argc(argc), m_argv(argv), m_global(NULL)
 {
-    Filesystem::convert_native_arguments (m_argc, m_argv);
 }
 
 
@@ -350,8 +349,6 @@ ArgParse::parse (int xargc, const char **xargv)
     m_argc = xargc;
     m_argv = xargv;
 
-    Filesystem::convert_native_arguments (m_argc, m_argv);
-
     for (int i = 1; i < m_argc; i++) {
         if (m_argv[i][0] == '-' && 
               (isalpha (m_argv[i][1]) || m_argv[i][1] == '-')) {     // flag
@@ -532,7 +529,7 @@ ArgParse::usage () const
                     std::cout << std::string (maxlen + 2 - fmtlen, ' ');
                 else
                     std::cout << "\n    " << std::string (maxlen + 2, ' ');
-                std::cout << Strutil::wordwrap(opt->description(), columns-2, maxlen+2+4+2) << '\n';
+                std::cout << Strutil::wordwrap(opt->description(), columns-2, (int)maxlen+2+4+2) << '\n';
             }
         }
     }
