@@ -58,11 +58,32 @@
 #include "dassert.h"
 
 #include "strutil.h"
+#include "ustring.h"
+#include "string_ref.h"
 
 
 OIIO_NAMESPACE_ENTER
 {
 
+
+const char *
+string_ref::c_str() const
+{
+    // Usual case: either empty, or null-terminated
+    if (m_len == 0)   // empty string
+        return "";
+    else if (m_chars[m_len-1] == 0)  // 0-terminated
+        return m_chars;
+    // Rare case: may not be 0-terminated. Bite the bullet and construct a
+    // 0-terminated string.  We use ustring as a way to avoid any issues of
+    // who cleans up the allocation, though it means that it will stay in
+    // the ustring table forever. Punt on this for now, it's an edge case
+    // that we need to handle, but is not likely to ever be an issue.
+    return ustring(m_chars, 0, m_len).c_str();
+}
+
+
+
 std::string
 Strutil::format_raw (const char *fmt, ...)
 {
@@ -123,13 +144,13 @@ Strutil::vformat (const char *fmt, va_list ap)
 
 
 std::string
-Strutil::memformat (off_t bytes, int digits)
+Strutil::memformat (long long bytes, int digits)
 {
     const long long KB = (1<<10);
     const long long MB = (1<<20);
     const long long GB = (1<<30);
     const char *units = "B";
-    double d = bytes;
+    double d = (double)bytes;
     if (bytes >= GB) {
         units = "GB";
         d = (double)bytes / GB;
@@ -339,6 +360,34 @@ Strutil::iends_with (const char *a, const char *b)
 }
 
 
+bool
+Strutil::contains (const std::string &a, const std::string &b)
+{
+    return boost::algorithm::contains (a, b);
+}
+
+
+bool
+Strutil::contains (const char *a, const char *b)
+{
+    return boost::algorithm::contains (a, b);
+}
+
+
+bool
+Strutil::icontains (const std::string &a, const std::string &b)
+{
+    return boost::algorithm::icontains (a, b, loc);
+}
+
+
+bool
+Strutil::icontains (const char *a, const char *b)
+{
+    return boost::algorithm::icontains (a, b, loc);
+}
+
+
 void
 Strutil::to_lower (std::string &a)
 {
@@ -381,7 +430,7 @@ split_whitespace (const std::string &str, std::vector<std::string> &result,
         while (i < len && ! ::isspace(str[i]))
             i++;
         if (j < i) {
-            if (maxsplit-- <= 0)
+            if (--maxsplit <= 0)
                 break;
             result.push_back (str.substr(j, i - j));
             while (i < len && ::isspace(str[i]))
@@ -410,7 +459,7 @@ Strutil::split (const std::string &str, std::vector<std::string> &result,
     size_t i = 0, j = 0, len = str.size(), n = sep.size();
     while (i+n <= len) {
         if (str[i] == sep[0] && str.substr(i, n) == sep) {
-            if (maxsplit-- <= 0)
+            if (--maxsplit <= 0)
                 break;
             result.push_back (str.substr(j, i - j));
             i = j = i + n;
@@ -445,7 +494,7 @@ Strutil::utf8_to_utf16 (const std::string& str)
     std::wstring native;
     
     native.resize(MultiByteToWideChar (CP_UTF8, 0, str.c_str(), -1, NULL, 0));
-    MultiByteToWideChar (CP_UTF8, 0, str.c_str(), -1, &native[0], native.size());
+    MultiByteToWideChar (CP_UTF8, 0, str.c_str(), -1, &native[0], (int)native.size());
 
     return native;
 }
@@ -458,12 +507,31 @@ Strutil::utf16_to_utf8 (const std::wstring& str)
     std::string utf8;
 
     utf8.resize(WideCharToMultiByte (CP_UTF8, 0, str.c_str(), -1, NULL, 0, NULL, NULL));
-    WideCharToMultiByte (CP_UTF8, 0, str.c_str(), -1, &utf8[0], utf8.size(), NULL, NULL);
+    WideCharToMultiByte (CP_UTF8, 0, str.c_str(), -1, &utf8[0], (int)utf8.size(), NULL, NULL);
 
     return utf8;
 }
 #endif
 
 
+
+char *
+Strutil::safe_strcpy (char *dst, const char *src, size_t size)
+{
+    if (src) {
+        for (size_t i = 0;  i < size;  ++i) {
+            if (! (dst[i] = src[i]))
+                return dst;   // finished, and copied the 0 character
+        }
+        // If we got here, we have gotten to the maximum length, and still
+        // no terminating 0, so add it!
+        dst[size-1] = 0;
+    } else {
+        dst[0] = 0;
+    }
+    return dst;
+}
+
+
 }
 OIIO_NAMESPACE_EXIT
