@@ -46,11 +46,6 @@ OIIO_NAMESPACE_ENTER
 {
 
 #if 0
-// Use reader/writer locks
-typedef shared_mutex ustring_mutex_t;
-typedef shared_lock ustring_read_lock_t;
-typedef unique_lock ustring_write_lock_t;
-#elif 0
 // Use regular mutex
 typedef mutex ustring_mutex_t;
 typedef lock_guard ustring_read_lock_t;
@@ -111,54 +106,110 @@ static ustring ustring_force_make_unique_call("");
 
 
 
+namespace {
+// Definitions to let us access libc++ string internals.
+// See libc++ <string> file for details.
+
+#ifdef _LIBCPP_ALTERNATE_STRING_LAYOUT
+struct libcpp_string__long {
+    std::string::pointer   __data_;
+    std::string::size_type __size_;
+    std::string::size_type __cap_;
+};
+#if _LIBCPP_BIG_ENDIAN
+    enum {libcpp_string__long_mask  = 0x1ul};
+#else  // _LIBCPP_BIG_ENDIAN
+    enum {libcpp_string__long_mask  = ~(std::string::size_type(~0) >> 1)};
+#endif  // _LIBCPP_BIG_ENDIAN
+#else
+struct libcpp_string__long {
+    std::string::size_type __cap_;
+    std::string::size_type __size_;
+    std::string::pointer   __data_;
+};
+#if _LIBCPP_BIG_ENDIAN
+    enum {libcpp_string__long_mask  = ~(std::string::size_type(~0) >> 1)};
+#else  // _LIBCPP_BIG_ENDIAN
+    enum {libcpp_string__long_mask  = 0x1ul};
+#endif  // _LIBCPP_BIG_ENDIAN
+#endif
+
+enum {libcpp_string__min_cap = (sizeof(libcpp_string__long) - 1)/sizeof(std::string::value_type) > 2 ?
+                               (sizeof(libcpp_string__long) - 1)/sizeof(std::string::value_type) : 2};
+
+};
+
+
+
 ustring::TableRep::TableRep (const char *s, size_t len)
     : hashed(Strutil::strhash(s))
 {
     strcpy ((char *)c_str(), s);
     length = len;
+
+    // We don't want the internal 'std::string str' to redundantly store the
+    // chars, along with our own allocation.  So we use our knowledge of the
+    // internal structure of std::string (for certain compilers) to force
+    // the std::string to make it point to our chars!  In such a case, the
+    // destructor will be careful not to allow a deallocation.
+
+#if defined(__GNUC__) && !defined(_LIBCPP_VERSION)
+    // It turns out that the first field of a gcc std::string is a pointer
+    // to the characters within the basic_string::_Rep.  We merely redirect
+    // that pointer, though for std::string to function properly, the chars
+    // must be preceeded immediately in memory by the rest of
+    // basic_string::_Rep, consisting of length, capacity and refcount
+    // fields.  And we have designed our TableRep to do just that!  So now
+    // we redirect the std::string's pointer to our own characters and its
+    // mocked-up _Rep.
+    //
+    // See /usr/include/c++/VERSION/bits/basic_string.h for the details of
+    // gcc's std::string implementation.
     dummy_capacity = len;
     dummy_refcount = 1;   // so it never frees
-
-#if defined(__GNUC__)
-    // We don't want the internal 'string str' to redundantly store the
-    // chars, along with our own allocation.  So we use our knowledge of
-    // the internal structure of gcc strings to make it point to our chars!
-    // Note that we've carefully structured the TableRep fields so they
-    // mimic a GCC basic_string::_Rep.
-    //
-    // It turns out that the first field of a gcc std::string is a
-    // pointer to the characters within the basic_string::_Rep.  We
-    // merely redirect that pointer, though for std::string to function
-    // properly, the chars must be preceeded immediately in memory by
-    // the rest of basic_string::_Rep, consisting of length, capacity
-    // and refcount fields.  And we have designed our TableRep to do
-    // just that!  So now we redirect the std::string's pointer to our
-    // own characters and its mocked-up _Rep.  
-    //
-    // See /usr/include/c++/VERSION/bits/basic_string.h for the details
-    // of gcc's std::string implementation.
-
     *(const char **)&str = c_str();
-    DASSERT (str.c_str() == c_str());
-#else
-    // Not gcc -- just assign the internal string.  This will result in
-    // double allocation for the chars.  If you care about that, do
-    // something special for your platform, much like we did for gcc
-    // above.  (Windows users, I'm talking to you.)
-    str = s;
+    DASSERT (str.c_str() == c_str() && str.size() == len);
+    return;
+
+#elif defined(_LIBCPP_VERSION)
+    // libc++ uses a different std::string representation than gcc.  For
+    // long char sequences, it's two size_t's (capacity & length) followed
+    // by the pointer to allocated characters. (Gory detail: see the
+    // definitions above for how it varies slightly with endianness and
+    // _LIBCPP_ALTERNATE_STRING_LAYOUT.)  For short enough sequences, it's a
+    // single byte length followed immediately by the chars (the total being
+    // the same size as the long string).  There's no savings of space or
+    // allocations to be had for short strings, so we just let those behave
+    // as normal.  But if it's going to make a long string (we can tell from
+    // the length), we construct it ourselves, forcing the pointer to point
+    // to the charcters in the TableRep we allocated.
+    if (len >= libcpp_string__min_cap /* it'll be a "long string" */) {
+        ((libcpp_string__long *)&str)->__cap_ = libcpp_string__long_mask | (len+1);
+        ((libcpp_string__long *)&str)->__size_ = len;
+        ((libcpp_string__long *)&str)->__data_ = (char *)c_str();
+        DASSERT (str.c_str() == c_str() && str.size() == len);
+        return;
+    }
 #endif
+
+    // Remaining cases - just assign the internal string.  This may result
+    // in double allocation for the chars.  If you care about that, do
+    // something special for your platform, much like we did for gcc and
+    // libc++ above. (Windows users, I'm talking to you.)
+    str = s;
 }
 
 
 
 ustring::TableRep::~TableRep ()
 {
-#if defined(__GNUC__)
-    // Doctor the string to be empty again before destroying.
-    ASSERT (str.c_str() == c_str());
-    std::string empty;
-    memcpy (&str, &empty, sizeof(std::string));
-#endif
+    if (str.c_str() == c_str()) {
+        // This is one of those cases where we've carefully doctored the
+        // string to point to our allocated characters.  To make a safe
+        // string destroy, now force it to look like an empty string.
+        std::string empty;
+        memcpy (&str, &empty, sizeof(std::string));
+    }
 }
 
 
@@ -214,9 +265,8 @@ ustring::make_unique (const char *str)
             table[result] = rep;
             ++ustring_stats_unique;
             ustring_stats_memory += size;
-#ifndef __GNUC__
-            ustring_stats_memory += len+1;  // non-GNU replicates the chars
-#endif
+            if (rep->c_str() != rep->str.c_str())
+                ustring_stats_memory += len+1;  // chars are replicated
             return result;
         } else {
             // use the one in the table, and we'll delete the new one we
@@ -252,7 +302,7 @@ ustring::getstats (bool verbose)
             << ", unique " << ustring_stats_unique
             << ", " << Strutil::memformat(ustring_stats_memory);
     }
-#ifdef DEBUG
+#ifndef NDEBUG
     // See if our hashing is pathological by checking if there are multiple
     // strings that ended up with the same hash.
     UstringTable &table (ustring_table());
