@@ -62,6 +62,31 @@ using namespace ImageBufAlgo;
 
 
 
+ImageRec::ImageRec (const std::string &name, int nsubimages,
+                    const int *miplevels, const ImageSpec *specs)
+    : m_name(name), m_elaborated(true),
+      m_metadata_modified(false), m_pixels_modified(true),
+      m_imagecache(NULL)
+{
+    int specnum = 0;
+    m_subimages.resize (nsubimages);
+    for (int s = 0;  s < nsubimages;  ++s) {
+        int nmips = miplevels ? miplevels[s] : 1;
+        m_subimages[s].m_miplevels.resize (nmips);
+        m_subimages[s].m_specs.resize (nmips);
+        for (int m = 0;  m < nmips;  ++m) {
+            ImageBuf *ib = specs ? new ImageBuf (specs[specnum])
+                                 : new ImageBuf ();
+            m_subimages[s].m_miplevels[m].reset (ib);
+            if (specs)
+                m_subimages[s].m_specs[m] = specs[specnum];
+            ++specnum;
+        }
+    }
+}
+
+
+
 ImageRec::ImageRec (ImageRec &img, int subimage_to_copy,
                     int miplevel_to_copy, bool writable, bool copy_pixels)
     : m_name(img.name()), m_elaborated(true),
@@ -85,7 +110,7 @@ ImageRec::ImageRec (ImageRec &img, int subimage_to_copy,
             ImageBuf *ib = NULL;
             if (writable || img.pixels_modified() || !copy_pixels) {
                 // Make our own copy of the pixels
-                ib = new ImageBuf (img.name(), srcspec);
+                ib = new ImageBuf (srcspec);
                 if (copy_pixels)
                     ib->copy_pixels (srcib);
             } else {
@@ -103,6 +128,65 @@ ImageRec::ImageRec (ImageRec &img, int subimage_to_copy,
 
 
 
+ImageRec::ImageRec (ImageRec &A, ImageRec &B, int subimage_to_copy,
+                    WinMerge pixwin, WinMerge fullwin,
+                    TypeDesc pixeltype)
+    : m_name(A.name()), m_elaborated(true),
+      m_metadata_modified(false), m_pixels_modified(false),
+      m_imagecache(A.m_imagecache)
+{
+    A.read ();
+    B.read ();
+    int subimages = (subimage_to_copy < 0) ? 
+                          std::min(A.subimages(), B.subimages()) : 1;
+    int first_subimage = clamp (subimage_to_copy, 0, subimages-1);
+    m_subimages.resize (subimages);
+    for (int s = 0;  s < subimages;  ++s) {
+        int srcsub = s + first_subimage;
+        m_subimages[s].m_miplevels.resize (1);
+        m_subimages[s].m_specs.resize (1);
+        const ImageBuf &Aib (A(srcsub));
+        const ImageBuf &Bib (B(srcsub));
+        const ImageSpec &Aspec (Aib.spec());
+        const ImageSpec &Bspec (Bib.spec());
+        ImageSpec spec = Aspec;
+        ROI Aroi = get_roi (Aspec), Aroi_full = get_roi_full (Aspec);
+        ROI Broi = get_roi (Bspec), Broi_full = get_roi_full (Bspec);
+        switch (pixwin) {
+        case WinMergeUnion :
+            set_roi (spec, roi_union (Aroi, Broi)); break;
+        case WinMergeIntersection :
+            set_roi (spec, roi_intersection (Aroi, Broi)); break;
+        case WinMergeA :
+            set_roi (spec, Aroi); break;
+        case WinMergeB :
+            set_roi (spec, Broi); break;
+        }
+        switch (fullwin) {
+        case WinMergeUnion :
+            set_roi_full (spec, roi_union (Aroi_full, Broi_full)); break;
+        case WinMergeIntersection :
+            set_roi_full (spec, roi_intersection (Aroi_full, Broi_full)); break;
+        case WinMergeA :
+            set_roi_full (spec, Aroi_full); break;
+        case WinMergeB :
+            set_roi_full (spec, Broi_full); break;
+        }
+        if (pixeltype != TypeDesc::UNKNOWN)
+            spec.set_format (pixeltype);
+        spec.nchannels = std::min (Aspec.nchannels, Bspec.nchannels);
+        spec.channelnames.resize (spec.nchannels);
+        spec.channelformats.clear ();
+
+        ImageBuf *ib = new ImageBuf (spec);
+
+        m_subimages[s].m_miplevels[0].reset (ib);
+        m_subimages[s].m_specs[0] = spec;
+    }
+}
+
+
+
 ImageRec::ImageRec (ImageBufRef img, bool copy_pixels)
     : m_name(img->name()), m_elaborated(true),
       m_metadata_modified(false), m_pixels_modified(false),
@@ -133,7 +217,7 @@ ImageRec::ImageRec (const std::string &name, const ImageSpec &spec,
         m_subimages[s].m_miplevels.resize (miplevels);
         m_subimages[s].m_specs.resize (miplevels);
         for (int m = 0;  m < miplevels;  ++m) {
-            ImageBuf *ib = new ImageBuf (name, spec);
+            ImageBuf *ib = new ImageBuf (spec);
             m_subimages[s].m_miplevels[m].reset (ib);
             m_subimages[s].m_specs[m] = spec;
         }
@@ -142,30 +226,6 @@ ImageRec::ImageRec (const std::string &name, const ImageSpec &spec,
 
 
 
-ImageRec::ImageRec (const std::string &name, int nsubimages,
-                    const int *miplevels, const ImageSpec *specs)
-    : m_name(name), m_elaborated(true),
-      m_metadata_modified(false), m_pixels_modified(true),
-      m_imagecache(NULL)
-{
-    int specnum = 0;
-    m_subimages.resize (nsubimages);
-    for (int s = 0;  s < nsubimages;  ++s) {
-        m_subimages[s].m_miplevels.resize (miplevels[s]);
-        m_subimages[s].m_specs.resize (miplevels[s]);
-        for (int m = 0;  m < miplevels[s];  ++m) {
-            ImageBuf *ib = specs ? new ImageBuf (name, specs[specnum])
-                                 : new ImageBuf (name);
-            m_subimages[s].m_miplevels[m].reset (ib);
-            if (specs)
-                m_subimages[s].m_specs[m] = specs[specnum];
-            ++specnum;
-        }
-    }
-}
-
-
-
 bool
 ImageRec::read ()
 {
@@ -181,6 +241,7 @@ ImageRec::read ()
     }
     m_subimages.resize (subimages);
 
+    bool allok = true;
     for (int s = 0;  s < subimages;  ++s) {
         int miplevels = 0;
         m_imagecache->get_image_info (uname, s, 0, u_miplevels,
@@ -200,7 +261,7 @@ ImageRec::read ()
             ImageBuf *ib = new ImageBuf (name(), m_imagecache);
             bool ok = ib->read (s, m, forceread,
                                 TypeDesc::FLOAT /* force float */);
-            ASSERT (ok);
+            allok &= ok;
             m_subimages[s].m_miplevels[m].reset (ib);
             m_subimages[s].m_specs[m] = ib->spec();
             // For ImageRec purposes, we need to restore a few of the
@@ -215,5 +276,5 @@ ImageRec::read ()
 
     m_time = Filesystem::last_write_time (name());
     m_elaborated = true;
-    return true;
+    return allok;
 }
