@@ -32,20 +32,26 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cmath>
+#include <cstring>
 #include <ctime>
 #include <iostream>
 #include <iterator>
 
 #include <OpenEXR/ImathMatrix.h>
 #include <OpenEXR/ImathVec.h>
+#include <OpenEXR/half.h>
+
+#include <boost/bind.hpp>
 
 #include "argparse.h"
 #include "imageio.h"
 #include "ustring.h"
 #include "imagebuf.h"
 #include "imagebufalgo.h"
+#include "imagebufalgo_util.h"
 #include "texture.h"
 #include "fmath.h"
+#include "filesystem.h"
 #include "sysutil.h"
 #include "strutil.h"
 #include "timer.h"
@@ -53,9 +59,11 @@
 
 OIIO_NAMESPACE_USING
 
-static std::vector<std::string> filenames;
+static std::vector<ustring> filenames;
 static std::string output_filename = "out.exr";
 static bool verbose = false;
+static int nthreads = 0;
+static int threadtimes = 0;
 static int output_xres = 512, output_yres = 512;
 static std::string dataformatname = "half";
 static float sscale = 1, tscale = 1;
@@ -66,6 +74,7 @@ static int anisotropic = -1;
 static int iters = 1;
 static int autotile = 0;
 static bool automip = false;
+static bool dedup = true;
 static bool test_construction = false;
 static bool test_gettexels = false;
 static bool test_getimagespec = false;
@@ -87,16 +96,22 @@ static bool nounmipped = false;
 static bool gray_to_rgb = false;
 static bool resetstats = false;
 static bool testhash = false;
+static bool wedge = false;
+static int ntrials = 1;
+static int testicwrite = 0;
 static Imath::M33f xform;
 void *dummyptr;
 
+typedef void (*Mapping2D)(int,int,float&,float&,float&,float&,float&,float&);
+typedef void (*Mapping3D)(int,int,Imath::V3f&,Imath::V3f&,Imath::V3f&,Imath::V3f &);
+
 
 
 static int
 parse_files (int argc, const char *argv[])
 {
     for (int i = 0;  i < argc;  i++)
-        filenames.push_back (argv[i]);
+        filenames.push_back (ustring(argv[i]));
     return 0;
 }
 
@@ -117,10 +132,11 @@ getargs (int argc, const char *argv[])
                   "-o %s", &output_filename, "Output test image",
                   "-d %s", &dataformatname, "Set the output data format to one of:"
                         "uint8, sint8, uint10, uint12, uint16, sint16, half, float, double",
-                  "-res %d %d", &output_xres, &output_yres,
+                  "--res %d %d", &output_xres, &output_yres,
                       "Resolution of output test image",
-                  "-iters %d", &iters,
+                  "--iters %d", &iters,
                       "Iterations for time trials",
+                  "--threads %d", &nthreads, "Number of threads (default 0 = #cores)",
                   "--blur %f", &sblur, "Add blur to texture lookup",
                   "--stblur %f %f", &sblur, &tblur, "Add blur (s, t) to texture lookup",
                   "--width %f", &width, "Multiply filter width of texture lookup",
@@ -144,6 +160,7 @@ getargs (int argc, const char *argv[])
                   "--offset %f %f %f", &offset[0], &offset[1], &offset[2], "Offset texture coordinates",
                   "--scalest %f %f", &sscale, &tscale, "Scale texture lookups (s, t)",
                   "--cachesize %f", &cachesize, "Set cache size, in MB",
+                  "--nodedup %!", &dedup, "Turn off de-duplication",
                   "--scale %f", &scalefactor, "Scale intensities",
                   "--maxfiles %d", &maxfiles, "Set maximum open files",
                   "--nountiled", &nountiled, "Reject untiled images",
@@ -151,6 +168,10 @@ getargs (int argc, const char *argv[])
                   "--graytorgb", &gray_to_rgb, "Convert gratscale textures to RGB",
                   "--resetstats", &resetstats, "Print and reset statistics on each iteration",
                   "--testhash", &testhash, "Test the tile hashing function",
+                  "--threadtimes %d", &threadtimes, "Do thread timings (arg = workload profile)",
+                  "--trials %d", &ntrials, "Number of trials for timings",
+                  "--wedge", &wedge, "Wedge test",
+                  "--testicwrite %d", &testicwrite, "Test ImageCache write ability (1=seeded, 2=generated)",
                   NULL);
     if (ap.parse (argc, argv) < 0) {
         std::cerr << ap.geterror() << std::endl;
@@ -172,6 +193,26 @@ getargs (int argc, const char *argv[])
 
 
 
+static void
+initialize_opt (TextureOpt &opt, int nchannels)
+{
+    opt.sblur = sblur;
+    opt.tblur = tblur >= 0.0f ? tblur : sblur;
+    opt.rblur = sblur;
+    opt.swidth = width;
+    opt.twidth = width;
+    opt.rwidth = width;
+    opt.nchannels = nchannels;
+    opt.fill = (fill >= 0.0f) ? fill : 1.0f;
+    if (missing[0] >= 0)
+        opt.missingcolor = (float *)&missing;
+    TextureOpt::parse_wrapmodes (wrapmodes.c_str(), opt.swrap, opt.twrap);
+    opt.rwrap = opt.swrap;
+    opt.anisotropic = anisotropic;
+}
+
+
+
 static void
 test_gettextureinfo (ustring filename)
 {
@@ -247,7 +288,7 @@ adjust_spec (ImageSpec &outspec, const std::string &dataformatname)
 
 
 inline Imath::V3f
-warp (float x, float y, Imath::M33f &xform)
+warp (float x, float y, const Imath::M33f &xform)
 {
     Imath::V3f coord (x, y, 1.0f);
     coord *= xform;
@@ -257,7 +298,7 @@ warp (float x, float y, Imath::M33f &xform)
 
 
 inline Imath::V3f
-warp (float x, float y, float z, Imath::M33f &xform)
+warp (float x, float y, float z, const Imath::M33f &xform)
 {
     Imath::V3f coord (x, y, z);
     coord *= xform;
@@ -266,6 +307,17 @@ warp (float x, float y, float z, Imath::M33f &xform)
 }
 
 
+inline Imath::V2f
+warp_coord (float x, float y)
+{
+    Imath::V3f coord = warp (x/output_xres, y/output_yres, xform);
+    coord.x *= sscale;
+    coord.y *= tscale;
+    coord += offset;
+    return Imath::V2f (coord.x, coord.y);
+}
+
+
 
 // Just map pixels to [0,1] st space
 static void
@@ -286,18 +338,9 @@ static void
 map_warp (int x, int y, float &s, float &t,
           float &dsdx, float &dtdx, float &dsdy, float &dtdy)
 {
-    Imath::V3f coord = warp (float(x+0.5f)/output_xres, float(y+0.5f)/output_yres, xform);
-    coord.x *= sscale;
-    coord.y *= tscale;
-    coord += offset;
-    Imath::V3f coordx = warp (float(x+1.5f)/output_xres, float(y+0.5f)/output_yres, xform);
-    coordx.x *= sscale;
-    coordx.y *= tscale;
-    coordx += offset;
-    Imath::V3f coordy = warp (float(x+0.5f)/output_xres, float(y+1.5f)/output_yres, xform);
-    coordy.x *= sscale;
-    coordy.y *= tscale;
-    coordy += offset;
+    const Imath::V2f coord  = warp_coord (x+0.5f, y+0.5f);
+    const Imath::V2f coordx = warp_coord (x+1.5f, y+0.5f);
+    const Imath::V2f coordy = warp_coord (x+0.5f, y+1.5f);
     s = coord[0];
     t = coord[1];
     dsdx = coordx[0] - coord[0];
@@ -319,21 +362,21 @@ map_tube (int x, int y, float &s, float &t,
     float theta = atan2f (yt, xt);
     // See OSL's Dual2 for partial derivs of
     // atan2, hypot, and 1/x
-    float denom = 1.0f / (xt*xt + yt*yt);
-    float dtheta_dx = yt*dxt_dx * denom;
-    float dtheta_dy = -xt*dyt_dy * denom;
-    s = 4.0f * theta / (2.0f * M_PI);
-    dsdx = 4.0f * dtheta_dx / (2.0f * M_PI);
-    dsdy = 4.0f * dtheta_dy / (2.0f * M_PI);
-    float h = hypot(xt,yt);
-    float dh_dx = xt*dxt_dx / h;
-    float dh_dy = yt*dyt_dy / h;
+    double denom = 1.0 / (xt*xt + yt*yt);
+    double dtheta_dx = yt*dxt_dx * denom;
+    double dtheta_dy = -xt*dyt_dy * denom;
+    s = float(4.0 * theta / (2.0 * M_PI));
+    dsdx = float(4.0 * dtheta_dx / (2.0 * M_PI));
+    dsdy = float(4.0 * dtheta_dy / (2.0 * M_PI));
+    double h = hypot(xt,yt);
+    double dh_dx = xt*dxt_dx / h;
+    double dh_dy = yt*dyt_dy / h;
     h *= M_SQRT2;
     dh_dx *= M_SQRT2; dh_dy *= M_SQRT2;
-    float hinv = 1.0f / h;
-    t = hinv;
-    dtdx = hinv * (-hinv * dh_dx);
-    dtdy = hinv * (-hinv * dh_dy);
+    double hinv = 1.0 / h;
+    t = float(hinv);
+    dtdx = float(hinv * (-hinv * dh_dx));
+    dtdy = float(hinv * (-hinv * dh_dy));
 }
 
 
@@ -357,7 +400,7 @@ map_filtertest (int x, int y, float &s, float &t,
 {
     float minoraxis = 1.0f/256;
     float majoraxis = minoraxis * lerp (1.0f, 32.0f, (float)x/(output_xres-1));
-    float angle = 2.0f * M_PI * (float)y/(output_yres-1);
+    float angle = (float)(2.0 * M_PI * (double)y/(output_yres-1));
     float sinangle, cosangle;
     sincos (angle, &sinangle, &cosangle);
     s = 0.5f;
@@ -420,242 +463,139 @@ map_warp_3D (int x, int y, Imath::V3f &P,
 
 
 
-template<class MAPPING>
 void
-test_plain_texture (MAPPING mapping)
+plain_tex_region (ImageBuf &image, ustring filename, Mapping2D mapping,
+                  ROI roi)
+{
+    TextureSystem::Perthread *perthread_info = texsys->get_perthread_info ();
+    TextureSystem::TextureHandle *texture_handle = texsys->get_texture_handle (filename);
+    int nchannels = image.nchannels();
+
+    TextureOpt opt;
+    initialize_opt (opt, nchannels);
+
+    float *result = ALLOCA (float, nchannels);
+    for (ImageBuf::Iterator<float> p (image, roi);  ! p.done();  ++p) {
+        float s, t, dsdx, dtdx, dsdy, dtdy;
+        mapping (p.x(), p.y(), s, t, dsdx, dtdx, dsdy, dtdy);
+
+        // Call the texture system to do the filtering.
+        bool ok;
+        if (use_handle)
+            ok = texsys->texture (texture_handle, perthread_info, opt,
+                                  s, t, dsdx, dtdx, dsdy, dtdy, result);
+        else
+            ok = texsys->texture (filename, opt,
+                                  s, t, dsdx, dtdx, dsdy, dtdy, result);
+        if (! ok) {
+            std::string e = texsys->geterror ();
+            if (! e.empty())
+                std::cerr << "ERROR: " << e << "\n";
+        }
+
+        // Save filtered pixels back to the image.
+        for (int i = 0;  i < nchannels;  ++i)
+            result[i] *= scalefactor;
+        image.setpixel (p.x(), p.y(), result);
+    }
+}
+
+
+
+void
+test_plain_texture (Mapping2D mapping)
 {
     std::cerr << "Testing 2d texture " << filenames[0] << ", output = " 
               << output_filename << "\n";
     const int nchannels = 4;
     ImageSpec outspec (output_xres, output_yres, nchannels, TypeDesc::HALF);
     adjust_spec (outspec, dataformatname);
-    ImageBuf image (output_filename, outspec);
+    ImageBuf image (outspec);
     ImageBufAlgo::zero (image);
 
-    Imath::M33f scale;  scale.scale (Imath::V2f (0.5, 0.5));
-    Imath::M33f rot;    rot.rotate (radians(30.0f));
-    Imath::M33f trans;  trans.translate (Imath::V2f (0.35f, 0.15f));
-    xform = scale * rot * trans;
-    xform.invert();
-
-    TextureOptions opt;
-    opt.sblur = sblur;
-    opt.tblur = tblur >= 0.0f ? tblur : sblur;
-    opt.swidth = width;
-    opt.twidth = width;
-    opt.nchannels = nchannels;
-    
-    float localfill = (fill >= 0.0f) ? fill : 1.0f;
-    opt.fill = localfill;
-    if (missing[0] >= 0)
-        opt.missingcolor.init ((float *)&missing, 0);
-//    opt.interpmode = TextureOptions::InterpSmartBicubic;
-//    opt.mipmode = TextureOptions::MipModeAniso;
-    TextureOptions::parse_wrapmodes (wrapmodes.c_str(), opt.swrap, opt.twrap);
-    opt.anisotropic = anisotropic;
-
-    TextureOpt opt1;
-    opt1.sblur = sblur;
-    opt1.tblur = tblur >= 0.0f ? tblur : sblur;
-    opt1.swidth = width;
-    opt1.twidth = width;
-    opt1.nchannels = nchannels;
-    opt1.fill = localfill;
-    if (missing[0] >= 0)
-        opt1.missingcolor = (float *)&missing;
-    TextureOpt::parse_wrapmodes (wrapmodes.c_str(), opt1.swrap, opt1.twrap);
-    opt1.anisotropic = anisotropic;
-
-    int shadepoints = blocksize*blocksize;
-    float *s = ALLOCA (float, shadepoints);
-    float *t = ALLOCA (float, shadepoints);
-    Runflag *runflags = ALLOCA (Runflag, shadepoints);
-    float *dsdx = ALLOCA (float, shadepoints);
-    float *dtdx = ALLOCA (float, shadepoints);
-    float *dsdy = ALLOCA (float, shadepoints);
-    float *dtdy = ALLOCA (float, shadepoints);
-    float *result = ALLOCA (float, shadepoints*nchannels);
-    
-    ustring filename = ustring (filenames[0]);
-    TextureSystem::Perthread *perthread_info = texsys->get_perthread_info ();
-    TextureSystem::TextureHandle *texture_handle = texsys->get_texture_handle (filename);
+    ustring filename = filenames[0];
 
     for (int iter = 0;  iter < iters;  ++iter) {
         if (iters > 1 && filenames.size() > 1) {
             // Use a different filename for each iteration
             int texid = std::min (iter, (int)filenames.size()-1);
-            filename = ustring (filenames[texid]);
+            filename = (filenames[texid]);
             std::cerr << "iter " << iter << " file " << filename << "\n";
         }
 
-        // Iterate over blocks
-        for (int by = 0, b = 0;  by < output_yres;  by+=blocksize) {
-            for (int bx = 0;  bx < output_xres;  bx+=blocksize, ++b) {
-                // Trick: switch to other textures on later iterations, if any
-                if (iters == 1 && filenames.size() > 1) {
-                    // Use a different filename from block to block
-                    filename = ustring (filenames[b % (int)filenames.size()]);
-                }
-
-                // Process pixels within a block.  First save the texture warp
-                // (s,t) and derivatives into SIMD vectors.
-                int idx = 0;
-                for (int y = by; y < by+blocksize; ++y) {
-                    for (int x = bx; x < bx+blocksize; ++x) {
-                        if (x < output_xres && y < output_yres) {
-                            mapping (x, y, s[idx], t[idx],
-                                     dsdx[idx], dtdx[idx], dsdy[idx], dtdy[idx]);
-                            runflags[idx] = RunFlagOn;
-                        } else {
-                            runflags[idx] = RunFlagOff;
-                        }
-                        ++idx;
-                    }
-                }
-
-                // Call the texture system to do the filtering.
-                bool ok;
-                if (blocksize == 1) {
-                    if (use_handle)
-                        ok = texsys->texture (texture_handle, perthread_info, opt1,
-                                              s[0], t[0], dsdx[0], dtdx[0],
-                                              dsdy[0], dtdy[0], result);
-                    else
-                        ok = texsys->texture (filename, opt1,
-                                              s[0], t[0], dsdx[0], dtdx[0],
-                                              dsdy[0], dtdy[0], result);
-                } else {
-                    ok = texsys->texture (filename, opt, runflags, 0,
-                                          shadepoints, Varying(s), Varying(t),
-                                          Varying(dsdx), Varying(dtdx),
-                                          Varying(dsdy), Varying(dtdy), result);
-                }
-                if (! ok) {
-                    std::string e = texsys->geterror ();
-                    if (! e.empty())
-                        std::cerr << "ERROR: " << e << "\n";
-                }
-                for (int i = 0;  i < shadepoints*nchannels;  ++i)
-                    result[i] *= scalefactor;
-
-                // Save filtered pixels back to the image.
-                idx = 0;
-                for (int y = by; y < by+blocksize; ++y) {
-                    for (int x = bx; x < bx+blocksize; ++x) {
-                        if (runflags[idx]) {
-                            image.setpixel (x, y, result + idx*nchannels);
-                        }
-                        ++idx;
-                    }
-                }
-            }
-        }
+        ImageBufAlgo::parallel_image (boost::bind(plain_tex_region, boost::ref(image), filename, mapping, _1),
+                                      get_roi(image.spec()), nthreads);
 
         if (resetstats) {
             std::cout << texsys->getstats(2) << "\n";
             texsys->reset_stats ();
         }
     }
-    
-    if (! image.save ()) 
+
+    if (! image.write (output_filename)) 
         std::cerr << "Error writing " << output_filename 
                   << " : " << image.geterror() << "\n";
 }
 
 
 
-template<class MAPPING>
 void
-test_texture3d (ustring filename, MAPPING mapping)
+tex3d_region (ImageBuf &image, ustring filename, Mapping3D mapping,
+              ROI roi)
+{
+    TextureSystem::Perthread *perthread_info = texsys->get_perthread_info ();
+    TextureSystem::TextureHandle *texture_handle = texsys->get_texture_handle (filename);
+    int nchannels = image.nchannels();
+
+    TextureOpt opt;
+    initialize_opt (opt, nchannels);
+    opt.fill = (fill >= 0.0f) ? fill : 0.0f;
+//    opt.swrap = opt.twrap = opt.rwrap = TextureOpt::WrapPeriodic;
+
+    float *result = ALLOCA (float, nchannels);
+    for (ImageBuf::Iterator<float> p (image, roi);  ! p.done();  ++p) {
+        Imath::V3f P, dPdx, dPdy, dPdz;
+        mapping (p.x(), p.y(), P, dPdx, dPdy, dPdz);
+
+        // Call the texture system to do the filtering.
+        bool ok = texsys->texture3d (texture_handle, perthread_info, opt, 
+                                     P, dPdx, dPdy, dPdz, result);
+        if (! ok) {
+            std::string e = texsys->geterror ();
+            if (! e.empty())
+                std::cerr << "ERROR: " << e << "\n";
+        }
+
+        // Save filtered pixels back to the image.
+        for (int i = 0;  i < nchannels;  ++i)
+            result[i] *= scalefactor;
+        image.setpixel (p.x(), p.y(), result);
+    }
+}
+
+
+
+void
+test_texture3d (ustring filename, Mapping3D mapping)
 {
     std::cerr << "Testing 3d texture " << filename << ", output = " 
               << output_filename << "\n";
     const int nchannels = 4;
     ImageSpec outspec (output_xres, output_yres, nchannels, TypeDesc::HALF);
     adjust_spec (outspec, dataformatname);
-    ImageBuf image (output_filename, outspec);
+    ImageBuf image (outspec);
     ImageBufAlgo::zero (image);
 
-    Imath::M33f scale;  scale.scale (Imath::V2f (0.5, 0.5));
-    Imath::M33f rot;    rot.rotate (radians(30.0f));
-    Imath::M33f trans;  trans.translate (Imath::V2f (0.35f, 0.15f));
-    xform = scale * rot * trans;
-    xform.invert();
-
-    TextureOptions opt;
-    opt.sblur = sblur;
-    opt.tblur = tblur >= 0.0f ? tblur : sblur;
-    opt.rblur = sblur;
-    opt.swidth = width;
-    opt.twidth = width;
-    opt.rwidth = width;
-    opt.nchannels = nchannels;
-    float localfill = (fill >= 0 ? fill : 0.0f);
-    opt.fill = localfill;
-    if (missing[0] >= 0)
-        opt.missingcolor.init ((float *)&missing, 0);
-
-    opt.swrap = opt.twrap = opt.rwrap = TextureOptions::WrapPeriodic;
-    opt.anisotropic = anisotropic;
-    int shadepoints = blocksize*blocksize;
-    Imath::V3f *P = ALLOCA (Imath::V3f, shadepoints);
-    Runflag *runflags = ALLOCA (Runflag, shadepoints);
-    Imath::V3f *dPdx = ALLOCA (Imath::V3f, shadepoints);
-    Imath::V3f *dPdy = ALLOCA (Imath::V3f, shadepoints);
-    Imath::V3f *dPdz = ALLOCA (Imath::V3f, shadepoints);
-    float *result = ALLOCA (float, shadepoints*nchannels);
-    
     for (int iter = 0;  iter < iters;  ++iter) {
-        // Iterate over blocks
-
         // Trick: switch to second texture, if given, for second iteration
         if (iter && filenames.size() > 1)
-            filename = ustring (filenames[1]);
+            filename = filenames[1];
 
-        for (int by = 0;  by < output_yres;  by+=blocksize) {
-            for (int bx = 0;  bx < output_xres;  bx+=blocksize) {
-                // Process pixels within a block.  First save the texture warp
-                // (s,t) and derivatives into SIMD vectors.
-                int idx = 0;
-                for (int y = by; y < by+blocksize; ++y) {
-                    for (int x = bx; x < bx+blocksize; ++x) {
-                        if (x < output_xres && y < output_yres) {
-                            mapping (x, y, P[idx], dPdx[idx], dPdy[idx], dPdz[idx]);
-                            runflags[idx] = RunFlagOn;
-                        } else {
-                            runflags[idx] = RunFlagOff;
-                        }
-                        ++idx;
-                    }
-                }
-                // Call the texture system to do the filtering.
-                bool ok = texsys->texture3d (filename, opt, runflags, 0, shadepoints,
-                                             Varying(P), Varying(dPdx),
-                                             Varying(dPdy), Varying(dPdz),
-                                             result);
-                if (! ok) {
-                    std::string e = texsys->geterror ();
-                    if (! e.empty())
-                        std::cerr << "ERROR: " << e << "\n";
-                }
-                for (int i = 0;  i < shadepoints*nchannels;  ++i)
-                    result[i] *= scalefactor;
-
-                // Save filtered pixels back to the image.
-                idx = 0;
-                for (int y = by; y < by+blocksize; ++y) {
-                    for (int x = bx; x < bx+blocksize; ++x) {
-                        if (runflags[idx]) {
-                            image.setpixel (x, y, result + idx*nchannels);
-                        }
-                        ++idx;
-                    }
-                }
-            }
-        }
+        ImageBufAlgo::parallel_image (boost::bind(tex3d_region, boost::ref(image), filename, mapping, _1),
+                                      get_roi(image.spec()), nthreads);
     }
     
-    if (! image.save ()) 
+    if (! image.write (output_filename)) 
         std::cerr << "Error writing " << output_filename 
                   << " : " << image.geterror() << "\n";
 }
@@ -695,7 +635,7 @@ test_getimagespec_gettexels (ustring filename)
     int w = spec.width / std::max(1,2<<miplevel);
     int h = spec.height / std::max(1,2<<miplevel);
     ImageSpec postagespec (w, h, spec.nchannels, TypeDesc::FLOAT);
-    ImageBuf buf ("postage.exr", postagespec);
+    ImageBuf buf (postagespec);
     TextureOptions opt;
     opt.nchannels = spec.nchannels;
     if (missing[0] >= 0)
@@ -712,7 +652,7 @@ test_getimagespec_gettexels (ustring filename)
             imagesize_t offset = (y*w + x) * spec.nchannels;
             buf.setpixel (x, y, &tmp[offset]);
         }
-    buf.save ();
+    buf.write ("postage.exr");
 }
 
 
@@ -825,18 +765,259 @@ test_hash ()
 
 
 
+static const char *workload_names[] = {
+    /*0*/ "None",
+    /*1*/ "Everybody accesses the same spot in one file (handles)",
+    /*2*/ "Everybody accesses the same spot in one file",
+    /*3*/ "Coherent access, one file, each thread in similar spots",
+    /*4*/ "Coherent access, one file, each thread in different spots",
+    /*5*/ "Coherent access, many files, each thread in similar spots",
+    /*6*/ "Coherent access, many files, each thread in different spots",
+    /*7*/ "Coherent access, many files, partially overlapping texture sets",
+    NULL
+};
+
+
+
+void
+do_tex_thread_workout (int iterations, int mythread)
+{
+    int nfiles = (int) filenames.size();
+    float s = 0.1f, t = 0.1f;
+    const int nchannels = 3;
+    float result[nchannels];
+    TextureOpt opt;
+    initialize_opt (opt, nchannels);
+    TextureSystem::Perthread *perthread_info = texsys->get_perthread_info ();
+    TextureSystem::TextureHandle *texture_handle = texsys->get_texture_handle (filenames[0]);
+    int pixel, whichfile = 0;
+    ImageSpec spec0;
+    texsys->get_imagespec (filenames[0], 0, spec0);
+    // Compute a filter size that's between the first and second MIP levels.
+    float fw = (1.0f / spec0.width) * 1.5f;
+    float fh = (1.0f / spec0.height) * 1.5f;
+    float dsdx = fw, dtdx = 0.0f, dsdy = 0.0f, dtdy = fh;
+
+    for (int i = 0;  i < iterations;  ++i) {
+        pixel = i;
+        bool ok = false;
+        // Several different texture access patterns
+        switch (threadtimes) {
+        case 1:
+            // Workload 1: Speed of light: Static texture access (same
+            // texture coordinates all the time, one file), with handles
+            // and per-thread data already queried only once rather than
+            // per-call.
+            ok = texsys->texture (texture_handle, perthread_info, opt, s, t,
+                                  dsdx, dtdx, dsdy, dtdy, result);
+            break;
+        case 2:
+            // Workload 2: Static texture access, with filenames.
+            ok = texsys->texture (filenames[0], opt, s, t,
+                                  dsdx, dtdx, dsdy, dtdy, result);
+            break;
+        case 3:
+        case 4:
+            // Workload 3: One file, coherent texture coordinates.
+            //
+            // Workload 4: Each thread starts with a different texture
+            // coordinate offset, so likely are not simultaneously
+            // accessing the very same tile as the other threads.
+            if (threadtimes == 4)
+                pixel += 57557*mythread;
+            break;
+        case 5:
+        case 6:
+            // Workload 5: Coherent texture coordinates, but access
+            // a series of textures at each coordinate.
+            //
+            // Workload 6: Each thread starts with a different texture
+            // coordinate offset, so likely are not simultaneously
+            // accessing the very same tile as the other threads.
+            whichfile = i % nfiles;
+            pixel = i / nfiles;
+            if (threadtimes == 6)
+                pixel += 57557*mythread;
+            break;
+        case 7:
+            // Workload 7: Coherent texture coordinates, but access
+            // a series of textures at each coordinate, which partially
+            // overlap with other threads.
+            {
+            int file = i % 8;
+            if (file < 2)        // everybody accesses the first 2 files
+                whichfile = std::min (file, nfiles-1);
+            else                 // and a slowly changing set of 6 others
+                whichfile = (file+11*mythread+i/1000) % nfiles;
+            pixel = i / nfiles;
+            pixel += 57557*mythread;
+            }
+            break;
+        default:
+            ASSERT_MSG (0, "Unkonwn thread work pattern %d", threadtimes);
+        }
+        if (! ok) {
+            s = (((2*pixel) % spec0.width) + 0.5f) / spec0.width;
+            t = (((2*((2*pixel) / spec0.width)) % spec0.height) + 0.5f) / spec0.height;
+            ok = texsys->texture (filenames[whichfile], opt, s, t,
+                                  dsdx, dtdx, dsdy, dtdy, result);
+        }
+        if (! ok) {
+            std::cerr << "Unexpected error: " << texsys->geterror() << "\n";
+            return;
+        }
+        // Do some pointless work, to simulate that in a real app, there
+        // would be operations interspersed with texture accesses.
+        for (int j = 0;  j < 30;  ++j)
+            for (int c = 0;  c < nchannels;  ++c)
+                result[c] = cosf (result[c]);
+    }
+    // Force the compiler to not optimize away the "other work"
+    for (int c = 0;  c < nchannels;  ++c)
+        ASSERT (! isnan(result[c]));
+}
+
+
+
+// Launch numthreads threads each of which performs a workout of texture
+// accesses.
+void
+launch_tex_threads (int numthreads, int iterations)
+{
+    texsys->invalidate_all (true);
+    boost::thread_group threads;
+    for (int i = 0;  i < numthreads;  ++i) {
+        threads.create_thread (boost::bind(do_tex_thread_workout,iterations,i));
+    }
+    ASSERT ((int)threads.size() == numthreads);
+    threads.join_all ();
+}
+
+
+
+class GridImageInput : public ImageInput {
+public:
+    GridImageInput () : m_miplevel(-1) { }
+    virtual ~GridImageInput () { close(); }
+    virtual const char * format_name (void) const { return "grid"; }
+    virtual bool valid_file (const std::string &filename) const { return true; }
+    virtual bool open (const std::string &name, ImageSpec &newspec) {
+        return seek_subimage (0, 0, newspec);
+    }
+    virtual bool close () { return true; }
+    virtual int current_miplevel (void) const { return m_miplevel; }
+    virtual bool seek_subimage (int subimage, int miplevel, ImageSpec &newspec) {
+        if (subimage > 0)
+            return false;
+        if (miplevel > 0 && automip /* if automip is on, don't generate MIP */)
+            return false;
+        if (miplevel == m_miplevel)
+            return true;
+        int res = 512;
+        res >>= miplevel;
+        if (res == 0)
+            return false;
+        m_spec = ImageSpec (res, res, 3, TypeDesc::FLOAT);
+        m_spec.tile_width = std::min (64, res);
+        m_spec.tile_height = std::min (64, res);
+        m_spec.tile_depth = 1;
+        newspec = m_spec;
+        m_miplevel = miplevel;
+        return true;
+    }
+    virtual bool read_native_scanline (int y, int z, void *data) { return false; }
+    virtual bool read_native_tile (int xbegin, int ybegin, int zbegin, void *data) {
+        float *tile = (float *)data;
+        for (int z = zbegin, zend = z+m_spec.tile_depth; z < zend; ++z)
+            for (int y = ybegin, yend = y+m_spec.tile_height; y < yend; ++y)
+                for (int x = xbegin, xend = x+m_spec.tile_width; x < xend; ++x) {
+                    tile[0] = float(x)/m_spec.width;
+                    tile[2] = float(y)/m_spec.height;
+                    tile[1] = (((x/16)&1) == ((y/16)&1)) ? 1.0f/(m_miplevel+1) : 0.05f;
+                    tile += m_spec.nchannels;
+                }
+        return true;
+    }
+private:
+    int m_miplevel;
+};
+
+
+
+ImageInput *make_grid_input () { return new GridImageInput; }
+
+
+
+void
+test_icwrite (int testicwrite)
+{
+    std::cout << "Testing IC write, mode " << testicwrite << "\n";
+
+    // The global "shared" ImageCache will be the same one the
+    // TextureSystem uses.
+    ImageCache *ic = ImageCache::create ();
+
+    // Set up the fake file ane add it
+    int tw = 64, th = 64;  // tile width and height
+    int nc = 3;  // channels
+    ImageSpec spec (512, 512, nc, TypeDesc::FLOAT);
+    spec.depth = 1;
+    spec.tile_width = tw;
+    spec.tile_height = th;
+    spec.tile_depth = 1;
+    ustring filename (filenames[0]);
+    bool ok = ic->add_file (filename, make_grid_input);
+    if (! ok)
+        std::cout << "ic->add_file error: " << ic->geterror() << "\n";
+    ASSERT (ok);
+
+    // Now add all the tiles if it's a seeded map
+    // testicwrite == 1 means to seed the first MIP level using add_tile.
+    // testicwrite == 2 does not use add_tile, but instead will rely on
+    // the make_grid_input custom ImageInput that constructs a pattern
+    // procedurally.
+    if (testicwrite == 1) {
+        std::vector<float> tile (spec.tile_pixels() * spec.nchannels);
+        for (int ty = 0;  ty < spec.height;  ty += th) {
+            for (int tx = 0;  tx < spec.width;  tx += tw) {
+                // Construct a tile
+                for (int y = 0; y < th; ++y)
+                    for (int x = 0; x < tw; ++x) {
+                        int index = (y*tw + x) * nc;
+                        int xx = x+tx, yy = y+ty;
+                        tile[index+0] = float(xx)/spec.width;
+                        tile[index+1] = float(yy)/spec.height;
+                        tile[index+2] = (!(xx%10) || !(yy%10)) ? 1.0f : 0.0f;
+                    }
+                bool ok = ic->add_tile (filename, 0, 0, tx, ty, 0, TypeDesc::FLOAT, &tile[0]);
+                if (! ok)
+                    std::cout << "ic->add_tile error: " << ic->geterror() << "\n";
+                ASSERT (ok);
+            }
+        }
+    }
+}
+
+
+
 int
 main (int argc, const char *argv[])
 {
+    Filesystem::convert_native_arguments (argc, argv);
     getargs (argc, argv);
 
+    OIIO::attribute ("threads", nthreads);
+
     texsys = TextureSystem::create ();
     std::cerr << "Created texture system\n";
     texsys->attribute ("statistics:level", 2);
     texsys->attribute ("autotile", autotile);
     texsys->attribute ("automip", (int)automip);
+    texsys->attribute ("deduplicate", (int)dedup);
     if (cachesize >= 0)
         texsys->attribute ("max_memory_MB", cachesize);
+    else
+        texsys->getattribute ("max_memory_MB", TypeDesc::TypeFloat, &cachesize);
     if (maxfiles >= 0)
         texsys->attribute ("max_open_files", maxfiles);
     if (searchpath.length())
@@ -864,12 +1045,15 @@ main (int argc, const char *argv[])
         std::cout << "TextureOpt memcpy: " << t() << " ns\n";
     }
 
+    if (testicwrite && filenames.size()) {
+        test_icwrite (testicwrite);
+    }
+
     if (test_getimagespec) {
         Timer t;
         ImageSpec spec;
-        ustring filename (filenames[0]);
         for (int i = 0;  i < iters;  ++i) {
-            texsys->get_imagespec (filename, 0, spec);
+            texsys->get_imagespec (filenames[0], 0, spec);
         }
         iters = 0;
     }
@@ -878,9 +1062,44 @@ main (int argc, const char *argv[])
         test_hash ();
     }
 
-    if (iters > 0 && filenames.size()) {
+    Imath::M33f scale;  scale.scale (Imath::V2f (0.5, 0.5));
+    Imath::M33f rot;    rot.rotate (radians(30.0f));
+    Imath::M33f trans;  trans.translate (Imath::V2f (0.35f, 0.15f));
+    xform = scale * rot * trans;
+    xform.invert();
+
+    if (threadtimes) {
+        const int iterations = 2000000;
+        std::cout << "Workload: " << workload_names[threadtimes] << "\n";
+        std::cout << "texture cache size = " << cachesize << " MB\n";
+        std::cout << "hw threads = " << boost::thread::hardware_concurrency() << "\n";
+        std::cout << "times are best of " << ntrials << " trials\n\n";
+        std::cout << "threads  time (s) efficiency\n";
+        std::cout << "-------- -------- ----------\n";
+
+        if (nthreads == 0)
+            nthreads = boost::thread::hardware_concurrency();
+        static int threadcounts[] = { 1, 2, 4, 6, 8, 10, 12, 16, 20, 24, 28, 32, 64, 128, 1024, 1<<30 };
+        float single_thread_time = 0.0f;
+        for (int i = 0; threadcounts[i] <= nthreads; ++i) {
+            int nt = wedge ? threadcounts[i] : nthreads;
+            int its = iterations; // / nt;
+            double range;
+            double t = time_trial (boost::bind(launch_tex_threads,nt,its),
+                                   ntrials, &range);
+            if (nt == 1)
+                single_thread_time = (float)t;
+            float efficiency = (single_thread_time /*/nt*/) / (float)t;
+            std::cout << Strutil::format ("%2d      %8.2f %6.1f%%    range %.2f\t(%d iters/thread)\n",
+                                          nt, t, efficiency*100.0f, range, its);
+            if (! wedge)
+                break;    // don't loop if we're not wedging
+        }
+        std::cout << "\n";
+
+    } else if (iters > 0 && filenames.size()) {
         ustring filename (filenames[0]);
-        test_gettextureinfo (filename);
+        test_gettextureinfo (filenames[0]);
         const char *texturetype = "Plain Texture";
         texsys->get_texture_info (filename, 0, ustring("texturetype"),
                                   TypeDesc::STRING, &texturetype);
@@ -908,7 +1127,7 @@ main (int argc, const char *argv[])
         }
         test_getimagespec_gettexels (filename);
     }
-    
+
     std::cout << "Memory use: "
               << Strutil::memformat (Sysutil::memory_used(true)) << "\n";
     TextureSystem::destroy (texsys);
