@@ -1,6 +1,7 @@
 #!/usr/bin/python 
 
 import os
+import glob
 import sys
 import platform
 import subprocess
@@ -44,13 +45,16 @@ parent = "../../../../../"
 
 command = ""
 outputs = [ "out.txt" ]    # default
+failureok = 0
+failthresh = 0.004
+failpercent = 0.02
 
 #print ("srcdir = " + srcdir)
 #print ("tmpdir = " + tmpdir)
 #print ("path = " + path)
 #print ("refdir = " + refdir)
 
-
+###########################################################################
 
 # Handy functions...
 
@@ -95,18 +99,20 @@ def oiio_app (app):
     # When we use Visual Studio, built applications are stored
     # in the app/$(OutDir)/ directory, e.g., Release or Debug.
     if (platform.system () != 'Windows' or options.devenv_config == ""):
-        return os.path.join (path, app, app) + " "
+        return os.path.join (path, "src", app, app) + " "
     else:
-        return os.path.join (path, app, options.devenv_config, app) + " "
+        return os.path.join (path, "src", app, options.devenv_config, app) + " "
 
 
-# Construct a command that will compare two images, appending output to
+# Construct a command that will print info for an image, appending output to
 # the file "out.txt".  If 'safematch' is nonzero, it will exclude printing
 # of fields that tend to change from run to run or release to release.
-def info_command (file, extraargs="", safematch=0) :
+def info_command (file, extraargs="", safematch=0, hash=True) :
     if safematch :
         extraargs += " --no-metamatch \"DateTime|Software|OriginatingProgram|ImageHistory\""
-    return (oiio_app("oiiotool") + "--info -v -a --hash " + extraargs
+    if hash :
+        extraargs += " --hash"
+    return (oiio_app("oiiotool") + "--info -v -a " + extraargs
             + " " + oiio_relpath(file,tmpdir) + " >> out.txt ;\n")
 
 
@@ -115,9 +121,12 @@ def info_command (file, extraargs="", safematch=0) :
 # 1 LSB (8 bit) error, it's very hard to make different platforms and
 # compilers always match to every last floating point bit.
 def diff_command (fileA, fileB, extraargs="", silent=False, concat=True) :
-    command = (oiio_app("idiff") + "-a "
-               + "-failpercent 0.01 -hardfail 0.004 -warn 0.004 "
-               + extraargs + " " + oiio_relpath(fileA,tmpdir) 
+    command = (oiio_app("idiff") + "-a"
+               + " -fail " + str(failthresh/4)
+               + " -failpercent " + str(failpercent)
+               + " -hardfail " + str(failthresh)
+               + " -warn " + str(2*failthresh)
+               + " " + extraargs + " " + oiio_relpath(fileA,tmpdir) 
                + " " + oiio_relpath(fileB,tmpdir))
     if not silent :
         command += " >> out.txt"
@@ -168,13 +177,23 @@ def rw_command (dir, filename, testwrite=1, use_oiiotool=0, extraargs="",
     return cmd
 
 
-# Construct a command that will test 
+# Construct a command that will testtex
 def testtex_command (file, extraargs="") :
     cmd = (oiio_app("testtex") + " " + file + " " + extraargs + " " +
            " >> out.txt ;\n")
     return cmd
 
 
+# Construct a command that will run oiiotool and append its output to out.txt
+def oiiotool (args, silent=False, concat=True) :
+    cmd = (oiio_app("oiiotool") + " " + args)
+    if not silent :
+        cmd += " >> out.txt"
+    if concat:
+        cmd += " ;\n"
+    return cmd
+
+
 
 # Run 'command'.  For each file in 'outputs', compare it to the copy
 # in 'ref/'.  If all outputs match their reference copies, return 0
@@ -185,10 +204,9 @@ def runtest (command, outputs, failureok=0) :
     os.chdir (srcdir)
     open ("out.txt", "w").close()    # truncate out.txt
 
-    assert options is not None
     if options.path != "" :
         sys.path = [options.path] + sys.path
-    print "command = " + command
+    print ("command = " + command)
 
     test_environ = None
     if (platform.system () == 'Windows') and (options.solution_path != "") and \
@@ -203,37 +221,44 @@ def runtest (command, outputs, failureok=0) :
     for sub_command in [c.strip() for c in command.split(';') if c.strip()]:
         cmdret = subprocess.call (sub_command, shell=True, env=test_environ)
         if cmdret != 0 and failureok == 0 :
-            print "#### Error: this command failed: ", sub_command
-            print "FAIL"
+            print ("#### Error: this command failed: ", sub_command)
+            print ("FAIL")
             return (1)
 
     err = 0
     for out in outputs :
         extension = os.path.splitext(out)[1]
-        if extension == ".tif" or extension == ".exr" :
-            # images -- use idiff
-            cmpcommand = diff_command (out, refdir + out, concat=False)
-            # print "cmpcommand = " + cmpcommand
-            cmpresult = os.system (cmpcommand)
-        elif extension == ".txt" :
-            cmpresult = text_diff (out, refdir + out, out + ".diff")
-        else :
-            # anything else
-            cmpresult = 0 if filecmp.cmp (out, refdir + out) else 1
-        
-        if cmpresult == 0 :
-            print "\tmatch " + out
-        else :
-            print "\tNO MATCH " + out
+        ok = 0
+        # We will first compare out to ref/out, and if that fails, we
+        # will compare it to everything else with the same extension in
+        # the ref directory.  That allows us to have multiple matching
+        # variants for different platforms, etc.
+        for testfile in (["ref/"+out] + glob.glob (os.path.join ("ref", "*"+extension))) :
+            # print ("comparing " + out + " to " + testfile)
+            if extension == ".tif" or extension == ".exr" or extension == ".jpg":
+                # images -- use idiff
+                cmpcommand = diff_command (out, testfile, concat=False)
+                # print ("cmpcommand = " + cmpcommand)
+                cmpresult = os.system (cmpcommand)
+            elif extension == ".txt" :
+                cmpresult = text_diff (out, testfile, out + ".diff")
+            else :
+                # anything else
+                cmpresult = 0 if filecmp.cmp (out, testfile) else 1
+            if cmpresult == 0 :
+                print ("PASS: " + out + " matches " + testfile)
+                ok = 1
+                break      # we're done
+
+        if ok == 0:
             err = 1
+            print ("NO MATCH for " + out)
+            print ("FAIL " + out)
 
-    if err == 0 :
-        print "PASS"
-    else :
-        print "FAIL"
     return (err)
 
 
+##########################################################################
 
 
 
@@ -241,7 +266,9 @@ def runtest (command, outputs, failureok=0) :
 # Read the individual run.py file for this test, which will define 
 # command and outputs.
 #
-execfile ("run.py")
+with open("run.py") as f:
+    code = compile(f.read(), "run.py", 'exec')
+    exec (code)
 
 # Run the test and check the outputs
 ret = runtest (command, outputs)
